#!/bin/sh
# \
exec wish "$0" {1+"$@"}

source "core_ops.tcl"
source "images.tcl"
source "readimg.tcl"
source "random.tcl"

set color blue

foreach i {1 2 3 4 5 6 7 8 9} { 
    readimg z$i imgz$i
    readimg p$i imgp$i
};  readimg sel imgsel

scrollbar .sbh -orient horiz -command {.c xview}
scrollbar .sbv -orient vert -command {.c yview}
canvas .c -confine yes -relief flat -bd 0 \
    -xscrollcommand {sbset .sbh} \
    -yscrollcommand {sbset .sbv} 

grid columnconfigure . 1 -weight 1
grid rowconfigure . 1 -weight 1
grid columnconfigure . 2 -weight 0
grid rowconfigure . 2 -weight 0
grid .sbh -row 2 -column 1 -sticky nsew
grid .sbv -row 1 -column 2 -sticky nsew
grid .c -row 1 -column 1 -sticky nsew

frame .top 
grid .top -row 0 -column 1 -columnspan 2 -sticky nsew
grid rowconfigure . 0 -weight 0

menu .root -tearoff 0
# . conf -menu .root

set itemlist {
    {submit} {Next Tile} {win_submit}
    {restart} {Reset Game} {win_new_game}
    {seed} {Random...} {display_random}
    {color} {Color...}  {color_dlg}
    {quit} {Quit}      {exit}
}
foreach {itmid itmlbl itmcmd} $itemlist {
    button .top.$itmid -text $itmlbl -command $itmcmd
    pack .top.$itmid -fill y -side left
    .root add command -label $itmlbl -command $itmcmd
}

foreach tgt $targetlist {
    set target_id($tgt) [ .c create polygon 0 1 1 0 3 0 4 1 3 2 1 2 \
        -fill white -outline gray -tags [list target $tgt] ]
    .c create text 2 1 -text $tgt -tags [list $tgt]
    .c create polygon 0 1 1 0 3 0 4 1 3 2 1 2 \
        -fill {} -outline {} -tags [list $tgt a$tgt seal] 

    target2xy $tgt xp yp;  .c move $tgt $xp $yp
    .c bind a$tgt <ButtonPress-1> "warpseltotgt $tgt; break"
}
foreach xyz $tilelist {
    foreach {x y z} [split $xyz {}] {break}
    .c create polygon 0 1 1 0 3 0 4 1 3 2 1 2 -fill $color \
            -tags [list d$xyz bg depot] 
    .c create image 2 1 -image p[lindex $dy $y] -tags [list d$xyz]
    .c create image 2 1 -image p[lindex $dz $z] -tags [list d$xyz]
    .c create image 2 1 -image p[lindex $dx $x] -tags [list d$xyz]
    .c create image 2 0.4 -image z[lindex $dx $x] -tags [list d$xyz]
    .c create image 3 1.3 -image z[lindex $dy $y] -tags [list d$xyz]
    .c create image 1 1.3 -image z[lindex $dz $z] -tags [list d$xyz]
    .c create polygon 0 1 1 0 3 0 4 1 3 2 1 2 -fill {} \
            -tags [list d$xyz a$xyz seal]
    .c bind a$xyz <ButtonPress-1> "pickup $xyz %x %y; break"
    .c bind a$xyz <B1-Motion> "drag $xyz %x %y; break"
    .c bind a$xyz <ButtonRelease-1> "drop $xyz %x %y; break"
    .c bind a$xyz <Double-ButtonPress-1> "warphome [list $xyz]; break"

    depot2xy $xyz xp yp; incr xp 18
    .c move d$xyz $xp $yp
} 
.c scale all 0 0 25 43
set depotbox [.c bbox depot]; .c dtag depot depot
set tgtbox [.c bbox target]

foreach {x1 y1 x2 y2} $depotbox {break}
.c create polygon $x1 $y1 $x2 $y1 $x2 $y2 $x1 $y2 -tags {top} \
    -fill {} -outline black
foreach {x1 y1 x2 y2} $tgtbox {break}
.c create rectangle $x1 $y1 $x2 $y2 -tags {bg} \
    -fill $color -outline black

.c raise seal top; .c lower bg
.c bind top <1> {warpselhome; break}


proc update_scroll {} { global curbbox
    set bbox [.c bbox all]
    if {$bbox != $curbbox} {
        .c conf -scrollregion $bbox
        set curbbox $bbox
    }
} 

proc win_new_game {} { global tilelist
    save_random; setsel
    global tilelist targetlist
    foreach tile $tilelist {
        set tgt [target_of $tile]
	if {$tgt!={}} {
            unhighlighttarget $tgt
            removetile $tile
        }
        movehome $tile
    }
}

proc color_dlg {} { global color
    set newcolor [tk_chooseColor -initialcolor $color]
    .c itemconf bg -fill [set color $newcolor]
}

proc win_submit {} { global cursel
    if {$cursel=={} || [target_of $cursel]!={}} {
        setsel [random_tile]
    }
}

proc random_tile {} { global tilelist
    set tile {}
    while {$tile=={} || [target_of $tile]!={}} {
        set tile [lindex $tilelist [random 27]]
    }
    finish_random ;# at least for a while :-)
    return $tile
}

proc getpos {xyz xn yn} { upvar 1 $xn x $yn y
    # anchor is topleft-point of frame (x=1st and y=4th coord of polygon)
    foreach {x d1 d2 y} [.c coords a$xyz] {break}
}
proc moveto {xyz x y} {
    getpos $xyz x0 y0
    .c move d$xyz [expr {$x - $x0}] [expr {$y - $y0}]
    # update idletasks
    update_scroll
}
proc movehome {xyz} {
    depot2xy $xyz xp yp
    moveto $xyz [expr {($xp+18)*25}] [expr {$yp*43}]
}
proc movetarget {xyz tgt} {
    target2xy $tgt x y
    moveto $xyz [expr {$x*25}] [expr {$y*43}]
}

proc setsel {{xyz {}}} { global cursel 
    if {$xyz != $cursel} {
        if {$cursel != {}} {
            .c delete sel
        }
        if {$xyz != {}} {
            getpos $xyz x y
            .c create image $x $y -image sel -tags [list sel d$xyz] -anchor nw
            .c lower d$xyz top; .c raise a$xyz
        }
    }
    set cursel $xyz
}
proc warpselhome {} { global cursel
    if {$cursel != {}} {
        warphome $cursel 
    }
}
proc warpseltotgt {tgt} { global cursel
    if {$cursel != {}} {
        warpto $cursel $tgt
    }
}
    
proc warpto {xyz tgt} {
    set tg [target_of $xyz]
    set ti [tile_at $tgt]

    if {$ti == {}} {
        if {$tg != {}} {
            unhighlighttarget $tg 
            removetile $xyz
        }
        placetile $xyz $tgt
        highlighttarget $tgt 
        .c lower d$xyz top; .c raise a$xyz
        update_score
        movetarget $xyz $tgt
    } elseif {$ti != $xyz} {
        bell; # target is not free
    }; # else: nothing to do 
}

proc warphome {xyz} {
    set tgt [target_of $xyz]
    if {$tgt != {}} {
        unhighlighttarget $tgt 
        removetile $xyz
        update_score
    }
    movehome $xyz
}
    
proc pickup {xyz x y} {
    global lastX lastY tiles curtgt
    .c lower d$xyz top; .c raise a$xyz
    setsel $xyz
    set lastX [.c canvasx $x]; set lastY [.c canvasy $y]

    set curtgt $tiles($xyz)
}

proc drag {xyz x y} {
    global lastX lastY curtgt 
    set x [.c canvasx $x]; set y [.c canvasy $y]
    .c move d$xyz [expr $x-$lastX] [expr $y-$lastY]
    set lastX $x; set lastY $y

    set newtgt [searchtarget $x $y]
    if {$newtgt!={}} {
        set t [tile_at $newtgt]
        if {$t!={} && $t!=$xyz} {set newtgt {}}
    }
    if {$newtgt!=$curtgt} {
        if {$curtgt!={}} { removetile $xyz; unhighlighttarget $curtgt }
        if {$newtgt!={}} { placetile $xyz $newtgt; highlighttarget $newtgt }
        update_score
    }; set curtgt $newtgt
}

proc drop {xyz x y} { global curtgt
    if {$curtgt!={}} {
        movetarget $xyz $curtgt
        set curtgt {}
    } else { update_scroll }
}
proc searchtarget {x y} {
    foreach item [.c find overlapping $x $y $x $y] {
        if {[lindex [.c gettags $item] 0] == "target"} {
            return [lindex [.c gettags $item] 1]
        }
    }
}
proc sbset {sb mi ma} {
    if {$mi==0 && $ma==1} {grid remove $sb} {grid configure $sb}
    $sb set $mi $ma
}
proc update_score {} {
    wm title . "Take It Easy - Score: [count]"
}
proc highlighttarget {tgt} { global target_id
    .c itemconf $target_id($tgt) -stipple gray75
}
proc unhighlighttarget {tgt} { global target_id
    .c itemconf $target_id($tgt) -stipple {}
}
proc key_handler {k} { global cursel
    set used 1
    switch -- $k {
        0 { setsel }

        a - b - f - t - x - y - z { bell }
        c - d - e - g - h - i - j - k - l - m - 
            n - o - p - q - r - s - u - v - w {
            if {$cursel!={}} { warpto $cursel [string toupper $k] }
        }

        Q - Esc { exit }
        R { win_new_game }
        S { display_random }
        P { global tiles target; parray tiles; parray target }
        C { color_dlg }
        space { warpselhome }
        Return { win_submit }
        default { set used 0}
    }
}

set curbbox {}
set curtgt {}
set cursel {}

# bind .c <2> {.c scan mark %x %y; break}
# bind .c <B2-Motion> {.c scan dragto %x %y; break}
bind .c <3> {tk_popup .root %X %Y; break}
bind .c <Shift-1> {tk_popup .root %X %Y; break}

bind . <Up> {.c yview scroll -1 units; break}
bind . <Down> {.c yview scroll 1 units; break}
bind . <Left> {.c xview scroll -1 units; break}
bind . <Right> {.c xview scroll 1 units; break}
bind . <Key> {key_handler %K; break}

foreach {xs ys xe ye} [.c bbox all] {break}
.c conf -width [expr {$xe-$xs}] -height [expr {$ye-$ys}]

update_score 
update_scroll
save_random; finish_random


