#!/usr/bin/tclsh

# run this in the  xpms subdirectoy of the game
#  (that is: the directory where this script resides in)

if {[file isdirectory xpms]} {puts "Entering images directory"; cd xpms}

proc collect {fncoll lfnxpm} {
    set outf [open $fncoll "w"]
    foreach fnxpm $lfnxpm {
        regsub {\.xpm$} $fnxpm {} imgname
        set inf [open $fnxpm "r"]
        set xpmdata [read $inf]; close $inf
        set imgdata [convert xpmdata]
	puts $outf [list set [list "img$imgname"] $imgdata]
        puts $imgname
    } 
    close $outf
}

proc convert {dataname args} { upvar 1 $dataname data
    set colcode "c"
    foreach {opt val} $args {
        switch -exact -- $opt {
            -code {set colcode $val}
            default {error "Invalid option. valid option: -code."}
        }
    }
    
    set state gethdr; set row 0; set img {}
    foreach line [split $data "\n"] {
        set s [string first "\"" $line]
        set e [string last "\"" $line]
        if {$s==-1 || $e==-1 || $s==$e} continue
        incr s; incr e -1; set line [string range $line $s $e]
        switch -exact -- $state {
            gethdr {
                foreach {w h cols cpp} $line {}
                set state getcols
            }
            getcols {
                set colstr [string range $line 0 [expr {$cpp-1}]]
                set line [string range $line $cpp end]
                set col {}
                foreach {code col} $line {
                    if {$code==$colcode} {set colval $col; break}
                }
                if {$col=={}} {
                    error "$dataname: colortable has no $colcode-field"
                }
                if {[string tolower $col] == "none"} {set col {}}
                set coltable($colstr) $col

                incr cols -1; if {$cols==0} {set state getpix}
            }
            getpix {
                set s 0; set e $cpp; incr e -1
                set ii -1; set clist {}
                for {set i 0} {$i < $w} {incr i} {
                    set c $coltable([string range $line $s $e])
                    incr s $cpp; incr e $cpp
                    if {$c=={}} {
                        if {$ii>=0} {
                            lappend img [list $ii $row [list $clist]]
                            set ii -1; set clist {}
                        }
                    } else {
                        lappend clist $c; if {$ii<0} {set ii $i}
                    }
                }
                if {$ii >=0} {
                    lappend img [list $ii $row [list $clist]]
                    set ii -1
                }
                incr row
            }
            default {}
        }
    }
    return [list $w $h $img]
}

puts "converting ..."
collect [file join .. images.tcl] [glob *.xpm]

